use leptos::*;
use leptos_icons::*;
use leptos_router::*;
use crate::app::{
    models::Post,
    components::{
        Title, Loading
    }
};

#[server]
pub async fn get_posts(
    tag: Option<String>
) -> Result<Vec<Post>, ServerFnError> {
    leptos_actix::extract(
        |data: actix_web::web::Data<crate::app::models::Data>| async move {
            let data = data.into_inner();
            let default = vec![];
            let posts = match tag {
                Some(tag) => data.posts_by_tag.get(&tag).unwrap_or(&default),
                None => &data.posts
            };
            posts.iter()
                .map(|post| {
                    Post {
                        metadata: post.metadata.clone(),
                        content: post.content.clone(),
                    }
                })
                .collect::<Vec<_>, >()
        },
    ).await
}

#[server]
pub async fn get_post(
    slug: String
) -> Result<Post, ServerFnError> {
    leptos_actix::extract(
        |data: actix_web::web::Data<crate::app::models::Data>| async move {
            let data = data.into_inner();
            data.posts_by_slug.get(&slug)
                .and_then(|post| Some(Post {
                    metadata: post.metadata.clone(),
                    content: post.content.clone(),
                }))
        },
    )
    .await
    .and_then(|post| post.ok_or_else(|| ServerFnError::ServerError("Post not found".to_string())))
}

#[component]
pub fn PostTags(
    tags: Vec<String>
) -> impl IntoView {
    view! {
        <div class="tags_list">
        {
            tags.into_iter().map(|tag| view! { <A class="tag" href=format!("/posts?tag={}", tag)>{tag}</A>}).collect_view()
        }
        </div>
    }
}

#[component]
pub fn PostListCard(
    post: Post
) -> impl IntoView {
    view! {
        <div>
            <img src={post.metadata.image_path.clone()} alt=format!("Image {}", post.metadata.title)/>

            {
                if post.metadata.draft {
                    Some(view!{
                        <div class="warning">
                            <Icon icon=Icon::from(IoIcon::IoConstruct)/>
                        </div>
                    })
                } else {
                    None
                }
            }

            <div>
                <PostTags tags=post.metadata.tags.clone()/>
                <h2><A href=format!("/posts/{}", post.metadata.slug.clone())>{post.metadata.title.clone()}</A></h2>
                <p>{post.metadata.description.clone()}</p>
                <span>{post.metadata.date.clone()}</span>
            </div>
        </div>
    }
}

#[component]
pub fn PostList() -> impl IntoView {
    let query = use_query_map();
    let tag = move || query.with(|query| query.get("tag").cloned());
    let posts = create_resource(move || tag(), move |_| get_posts(tag()));

    let posts_view = move || {
        posts.and_then(|posts| {
            posts.iter()
                .map(|post| view! { <PostListCard post=post.clone() /> })
                .collect_view()
        })
    };

    let title = move || match tag() {
        Some(tag) => view! { <Title href="/posts".to_string() title=format!("Posts for {}", tag)/> },
        None => view! { <Title href="/".to_string() title="Posts".to_string()/> }
    };

    view! {
        <Suspense fallback=move || view! { <Loading title="Chargement des posts...".to_string() /> }>
            <main class="posts">
                { title }

                <div class="posts__cards">{posts_view}</div>
            </main>
        </Suspense>
    }
}

#[component]
pub fn PostElement() -> impl IntoView {
    let params = use_params_map();
    let slug = move || params.with(|params| params.get("slug").cloned().unwrap_or_default());

    let post = create_resource(|| (), move |_| get_post(slug()));

    let post_view = move || {
        post.and_then(|post| {
            view! {
                <>
                    <Title href="/posts".to_string() title=post.metadata.title.clone()/>
                    <PostTags tags=post.metadata.tags.clone()/>
                    {
                        if post.metadata.draft {
                            Some(view!{
                                <div class="bg-warning text-on_warning dark:bg-dark_warning dark:text-dart_on_warning rounded-md p-5">
                                    r#"
                                    L'article est en cours d'écriture. La formulation peut ne pas être exacte et les phrases peuvent contenir des fautes.
                                    "#
                                </div>
                            })
                        } else {
                            None
                        }
                    }
                    <div inner_html={post.content.clone()}></div>
                </>
            }
        })
    };

    view! {
        <Suspense fallback=move || view! { <Loading title="Chargement du post...".to_string() /> }>
            <main class="post">
                {post_view}
                <script>load();</script>
            </main>
        </Suspense>
    }
}